/*
 * Copyright (c) 2012 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*jslint regexp: true*/
/*global window, document, setTimeout, screen, history, console, tau */

/*global $, app, tizen, TemplateManager */

/**
 * @class Ui
 */
function Ui() {
    'use strict';
    return;
}

(function strict() { // strict mode wrapper
    'use strict';
    Ui.prototype = {

        templateManager: null,

        monthNames: ([
            'January', 'February', 'March', 'April', 'May', 'June',
            'July', 'August', 'September', 'October', 'November', 'December'
        ]),
        /**
         * UI module initialisation
         * @param {object} app
         */
        init: function UI_init(app) {
            this.app = app;
            this.templateManager = new TemplateManager();
            $(document).ready(this.domInit.bind(this));

            // init inner objects
            this.home.context = this;
            this.alarm.context = this;
            this.new_event.context = this;
        },

        /**
         * When DOM is ready, initialise it
         */
        domInit: function UI_domInit() {
            this.templateManager.loadToCache(
                ['home', 'alarm', 'new_event', 'event', 'all_day_event'],
                this.initPages.bind(this)
            );
            // Disable text selection
            $.mobile.tizen.disableSelection(document);
        },

        /**
         * Append pages to body and initialise them
         */
        initPages: function UI_initPages() {
            var pages = [];

            pages.push(this.templateManager.get('home'));
            pages.push(this.templateManager.get('alarm'));
            pages.push(this.templateManager.get('new_event'));

            $('body').append(pages.join(''));
            this.fixContentHeight();

            this.home.init();
            this.alarm.init();

            if (!app.ui.new_event.initialized) {
                app.ui.new_event.init();
            }

            $(':jqmData(role="tabbar")').first().on('vmouseup', 'a',
                function deactivateButton() {
                    $(this).removeClass($.mobile.activeBtnClass);
                });

            $('#new_event').on(
                'pagebeforeshow',
                function beforeNewEventShow() {
                    var self = app.ui.new_event;
                    self.updateDateFormat();
                    self.setStartDate();
                    self.setEndDate();
                    $('#add-event-btn').removeClass('disabled');
                    // hide the datetime selector before the page is shown
                    $('.ui-datefield-selected')
                        .removeClass('ui-datefield-selected');
                    $('.ui-popupwindow').hide();
                }
            );

            $(document).ready(function onReady() {
                var duration = $('#customDuration');
                if ($('input[type=radio]:checked').val() !== 'Yes') {
                    duration
                        .val('')
                        .addClass('ui-disabled')
                        .prop('disabled', true)
                        .data('placeholder', '0');
                }

                $('input[type=radio]').change(function onInputChange() {
                    if ($(this).val() === 'Yes') {
                        duration.removeClass('ui-disabled');
                        duration.prop('disabled', false);
                        duration.focus();
                    } else {
                        duration.blur();
                        duration.addClass('ui-disabled')
                            .data('placeholder', '0');
                        duration.prop('disabled', true);
                    }
                });

                $('#customDuration').on('blur', function onBlur() {
                    if ($('input[type=radio]:checked').val() === 'Yes') {
                        var value = Math.abs(parseInt($(this).val(), 10));
                        if (isNaN(value)) {
                            value = 0;
                        }
                        $(this).val(value);
                    }
                    $('input[type=radio]').checkboxradio('refresh');
                });
            });

            $(window).resize(function onResize() {
                if ($.mobile.activePage.attr('id') === 'new_alarm') {
                    var content = $('#new_alarm [data-role=content]'),
                        customDuration = $('#customDuration'),
                        size = customDuration.position().top +
                            customDuration.outerHeight();
                    content.scrollview('scrollTo', 0, -1 * size, 0);
                }
            });

            document.addEventListener('webkitvisibilitychange',
                function onVisibilityChange() {
                    var dateTimePicker1 = $('#demo-date-1'),
                        dateTimePicker2 = $('#demo-date-2'),
                        startDate = dateTimePicker1.attr('data-date'),
                        endDate = dateTimePicker2.attr('data-date'),
                        clearDateGMT = function clearDateGMT(dateString) {
                            return new Date(dateString.substring(0,
                                dateString.indexOf('GMT')));
                        };

                    app.ui.new_event.updateDateFormat(true);
                    if (document.webkitVisibilityState === 'visible') {
                        if ($.mobile.activePage.attr('id') === 'new_event' ||
                                $.mobile.activePage.attr('id') ===
                                    'new_alarm') {
                            if (app.eventId !== 0) {
                                app.model.eventExists(app.eventId, null,
                                    function goHome() {
                                        $.mobile.changePage('#home');
                                    });
                            }
                        }
                        // make datePicker value independent
                        // from timezone change in the background
                        if ($.mobile.activePage.attr('id') === 'new_event') {
                            dateTimePicker1.datetimepicker('value',
                                clearDateGMT(startDate));
                            dateTimePicker2.datetimepicker('value',
                                clearDateGMT(endDate));
                        }
                    }
                });

            $('input#customDuration').on('keypress', function onKeyPress(e) {
                if ((e.keyCode < 48) || (e.keyCode > 57)) {
                    e.stopImmediatePropagation();
                    e.preventDefault();
                    e.stopPropagation();
                }
            });

            $('input#customDuration').on('input keyup', function onKeyUp() {
                var val = parseInt($(this).val(), 10),
                    max = parseInt($(this).attr('max'), 10),
                    min = 0;

                if (val > max) {
                    $(this).val(max);
                } else if (val < min) {
                    $(this).val(min);
                }
            });

            $('input#customDuration').on('keydown', function onKeyDown(e) {
                if (e.keyCode === 13) {
                    $('input#customDuration').blur();
                }
            });

            window.addEventListener('tizenhwkey', function onTizenHwKey(e) {
                if (e.keyName === 'back') {
                    if ($.mobile.activePage.attr('id') === 'home') {
                        tizen.application.getCurrentApplication().exit();
                    } else if ($.mobile.activePage.attr('id') === 'new_event') {
                        $.mobile.changePage('#home');
                    } else {
                        history.back();
                        $('#customDuration').blur();
                    }
                }
            });

            $.mobile.changePage('#home', 'pop', false, true);
        },

        /**
         * Contains methods related to the #home page
         * @namespace
         */
        home: {
            init: function UI_home_init() {
                var app = this.context.app, self = this,
                    alarm = this.context.alarm,
                    alarmValue = -1;

                $('#demo-date-1, #demo-date-2').datetimepicker();

                $('#exitBtn').on('click',
                    function onExitBtnClick() {
                        if ($.mobile.activePage.attr('id') === 'home') {
                            app.exit();
                        }
                    });
                $('input:radio').checkboxradio();
                alarm.setValue(alarmValue);
                alarm.updateDurationLabel(alarmValue);

                // buttons in the events list
                $('#events_list').on(
                    'click',
                    '.remove_event_btn',
                    function onRemoveBtnClick() {
                        var eventId = $(this).parents('.event').data('eventid');
                        app.eventId = eventId;
                        app.ui.popup('Are you sure?', {
                            'No': function cancel() {
                                $('#popup').popup('close');
                            },
                            'Yes': function confirm() {
                                app.model.eventExists(
                                    eventId,
                                    function onExists() {
                                        try {
                                            app.model.deleteEvent(eventId);
                                            $('#popup').popup('close');
                                        } catch (e) {
                                            app.ui.popup(e);
                                        }
                                    },
                                    function onNotExists() {
                                        app.ui.popup(
                                            'Selected event does not exist.'
                                        );
                                    }
                                );
                            }
                        });
                    }
                );

                $('#events_list').on(
                    'click',
                    '.edit_event_btn',
                    function onEditClick() {
                        var eventId = $(this).parents('.event').data('eventid'),
                            event = app.model.editEvent(eventId),
                            field,
                            date,
                            duration,
                            properties = ['summary', 'startDate', 'endDate'];

                        $('#demo-date-1').datetimepicker();
                        $('#demo-date-2').datetimepicker();
                        app.eventId = eventId;
                        properties.forEach(function each(element) {
                            if (event.hasOwnProperty(element)) {
                                field = $(
                                    '#new_event input[name="' + element + '"]'
                                );

                                if (field.length !== 0) {
                                    if (field.attr('type') === 'datetime') {
                                        date = self.TZD2Date(event[element]);
                                        field.datetimepicker('value', date);
                                    } else {
                                        field.val(event[element]);
                                    }
                                }
                            }
                        });

                        $('#new_event h1').text('Edit event');

                        if (event.alarms.length !== 0) {
                            if (event.alarms[0].before !== null) {
                                duration = app.retrieveTimeDurationInMinutes(
                                    event.alarms[0].before
                                );
                            } else {
                                duration = 0;
                            }
                        }

                        if (duration === undefined) {
                            duration = -1;
                        }
                        alarm.setValue(duration);
                        alarm.updateDurationLabel(duration);

                        $.mobile.changePage('#new_event');
                        // set select allDay property
                        app.ui.new_event.setSelectAllDay(event.isAllDay);
                        app.ui.new_event.updateDateFormat();
                    }
                ); // end on edit button click

                $('#newEventBtn').on('click', function onNewEventBtnClick() {
                    app.eventId = 0;
                    $('#demo-date-1').datetimepicker('value', new Date());
                    $('#demo-date-2').datetimepicker('value', new Date());
                    $('#new_event h1').text('New event');
                    $('#title').val('');
                    app.ui.new_event.setSelectAllDay(false);
                    app.ui.new_event.updateDateFormat();
                    alarm.setValue(0);
                    alarm.updateDurationLabel(0);
                });

                $('#homeDateFilter').change(function onHomeDateFilterChange() {
                    $('#exitBtn').blur();
                    app.homeDateFilter =
                        app.ui.new_event.getDateFromPicker($(this));
                    app.loadEvents();
                });

                this.loadEvents();
            }, // end init

            /**
             * Converts TZDate object to Date object.
             * @param {TZDate} tzdate
             * @return {Date}
             */
            TZD2Date: function TZD2Date(tzdate) {
                return new Date(
                    tzdate.getFullYear(),
                    tzdate.getMonth(),
                    tzdate.getDate(),
                    tzdate.getHours(),
                    tzdate.getMinutes(),
                    tzdate.getSeconds(),
                    tzdate.getMilliseconds()
                );
            },

            /**
             * Returns start date value from the form (#demo-date-1 field).
             *
             * @return {string}
             */
            getStartDate: function UI_home_getStartDate() {
                var startDate = $('#demo-date-1').attr('data-date');
                return startDate;
            },

            /**
             * Returns info if event is allDay event.
             *
             * @return {boolean}
             */
            getAllDayInfo: function UI_home_getAllDayInfo() {
                var isAllDay = $('select#allDay').val() === '1' ? true : false;
                return isAllDay;
            },
            /**
             * Gets end date value from the form (#demo-date-2 field).
             *
             * @return {string}
             */
            getEndDate: function UI_home_getEndDate() {
                var endDate = $('#demo-date-2').attr('data-date');
                return endDate;
            },
            /**
             * Gets the title from the form (#title field).
             *
             * @return {string}
             */
            getTitle: function UI_home_getTitle() {
                return $('#title').val();
            },
            /**
             * Gets the description from the form (#des field).
             *
             * @return {string}
             */
            getDescription: function UI_home_getDescription() {
                return $('#des').val();
            },
            /**
             * Gets the location from the form (#location field).
             *
             * @return {string}
             */
            getLocation: function UI_home_getLocation() {
                return $('#location').val();
            },
            /**
             * Wrapper for app.loadEvents.
             * @param {Date} date selected date
             */
            loadEvents: function UI_home_loadEvents(date) {
                this.context.app.loadEvents(date);
            },

            /**
             * Returns text for separating list items with events
             * Skips repeated values
             *
             * @param {CalendarEvent} event
             * @return {string}
             */
            getSeparatorText: function UI_home_getSeparatorText(event) {
                var previous = '', startDate, str;

                if (event === undefined) {
                    previous = '';
                    return undefined;
                }

                startDate = event.startDate;
                str = this.formatLongDate(startDate);

                if (previous === str) {
                    return ''; // skip it - already returned
                }
                // store in the closure for future comparison
                previous = str;

                return str;
            },

            /**
             * Format long date (D. MMMM YYYY)
             * @param {TZDate} date
             * @return {string}
             */
            formatLongDate: function UI_home_formatLongDate(date) {
                var month = app.ui.monthNames[date.getMonth()];
                return date.getDate() + ' ' + month + ' ' + date.getFullYear();
            },

            /**
             * Format short date (YYYY/MM/DD)
             * @param {TZDate} date
             * @return {string}
             */
            formatDate: function UI_home_formatDate(date) {
                return date.getFullYear() +
                        '/' + this.pad(date.getMonth() + 1) +
                        '/' + this.pad(date.getDate());
            },

            /**
             * Format date and time (YYYY/MM/DD hh:mm)
             * @param {TZDate} date
             * @return {string}
             */
            formatDateTime: function UI_home_formatDateTime(date) {
                return date.getFullYear() +
                    '/' + this.pad(date.getMonth() + 1) +
                    '/' + this.pad(date.getDate()) +
                    ' ' + this.pad(date.getHours()) +
                    ':' + this.pad(date.getMinutes());
            },

            /**
             * Zero-pads a positive number to 2 digits.
             * @param {Number} number
             */
            pad: function UI_home_pad(number) {
                return number < 10 ? '0' + number : number;
            },

            /**
             * Creates HTML representing the given array of alarms
             *
             * @param {Alarm[]} alarms
             * @return {string}
             */
            getAlarmsHtml: function UI_home_getAlarmsHtml(alarms) {
                var alarm = '', j, len;

                len = alarms.length;

                if (len) {
                    alarm += '<p class="ui-li-aside ui-li-desc">' +
                        '<img src="img/clock.png"/>';

                    for (j = 0; j < len; j += 1) {
                        if (alarms[j].before !== null) {
                            alarm += alarms[j].before.length;
                            alarm += ' ' + alarms[j].before.unit;
                        }
                    }
                    alarm += '</p>';
                }
                return alarm;
            },

            /**
             * Load the events into the #event_popup.
             *
             * Callback function for app.loadEvents.
             * @param {Array} events
             */
            onEventSearchSuccess:
                function UI_home_onEventSearchSuccess(events) {
                    var i = 0,
                        str = '',
                        event,
                        alarm = '',
                        dividerText = '',
                        templateParameters = {},
                        tmplName,
                        compareTZDates = function compareTZDates(a, b) {
                            if (a.getFullYear() !== b.getFullYear()) {
                                return a.getFullYear() < b.getFullYear() ?
                                        1 : -1;
                            }
                            if (a.getMonth() !== b.getMonth()) {
                                return a.getMonth() < b.getMonth() ? 1 : -1;
                            }
                            if (a.getDate() !== b.getDate()) {
                                return a.getDate() < b.getDate() ? 1 : -1;
                            }
                            return 0;
                        },
                        compareTZTimes = function compareTZTimes(a, b) {
                            if (a.getHours() !== b.getHours()) {
                                return a.getMinutes() < b.getMinutes() ? -1 : 1;
                            }
                            return a.getHours() < b.getHours() ? -1 : 1;
                        },
                        /**
                         *
                         * @param {CalendarEvent} a
                         * @param {CalendarEvent} b
                         * @return {Number}
                         */
                        compareEvents = function compareEvents(a, b) {
                            var result;
                            result = compareTZDates(a.startDate, b.startDate);
                            if (result !== 0) {
                                return result;
                            }
                            if (a.isAllDay !== b.isAllDay) {
                                return a.isAllDay ? 1 : -1;
                            }
                            result = compareTZTimes(a.startDate, b.startDate);
                            if (result !== 0) {
                                return result;
                            }
                            result = compareTZDates(a.endDate, b.endDate);
                            if (result !== 0) {
                                return result;
                            }
                            result = compareTZTimes(a.endDate, b.endDate);
                            if (result !== 0) {
                                return result;
                            }
                            if (a.name !== b.name) {
                                return a.name < b.name ? -1 : 1;
                            }
                            return 0;
                        },
                        endDate2;

                    events = events.sort(compareEvents);

                    // content
                    str = '';
                    if (events.length !== 0) {
                        for (i = 0; i < events.length; i += 1) {
                            event = events[i];

                            dividerText = this.getSeparatorText(event);

                            if (dividerText) {
                                str += '<li data-role="list-divider">' +
                                    dividerText + '</li>';
                            }

                            alarm = this.getAlarmsHtml(event.alarms);

                            endDate2 = new tizen.TZDate(
                                event.endDate.getFullYear(),
                                event.endDate.getMonth(),
                                event.endDate.getDate(),
                                0,
                                0,
                                0,
                                0
                            );

                            templateParameters = {
                                uid: event.id.uid,
                                startDateTime: this.formatDateTime(
                                    event.startDate
                                ),
                                endDateTime: this.formatDateTime(event.endDate),
                                startDate: this.formatDate(event.startDate),
                                endDate: this.formatDate(endDate2),
                                summary: event.summary || '[ no title ]',
                                location: event.location,
                                description: event.description,
                                alarm: alarm
                            };

                            tmplName = event.isAllDay ?
                                    'all_day_event' : 'event';
                            str += this.context.templateManager.get(tmplName,
                                templateParameters);
                        }
                    } else {
                        dividerText = this.getSeparatorText({
                            startDate: app.homeDateFilter
                        });

                        if (dividerText) {
                            str += '<li data-role="list-divider">' +
                                dividerText + '</li>';
                        }

                        str += '<li>No events found</li>';
                    }
                    this.getSeparatorText(); // clear the separator state

                    $('#events_list ul').html(str);
                    $('#events_list ul').listview();
                    $('#events_list ul').listview('refresh');
                    $('#events_list ul .edit_event_btn').button();
                    $('#events_list ul .remove_event_btn').button();

                    // scroll to top
                    $('#home [data-role=content]')
                        .scrollview('scrollTo', 100, 0, 0);
                },

            /**
             * Error handler for event search
             */
            onEventSearchError: function UI_home_onEventSearchError() {
                console.error('event search error');
            },

            updateHomeDateFilter:
                function UI_home_updateHomeDateFilter() {
                    $('#homeDateFilter')
                        .datetimepicker('value', app.homeDateFilter);
                }
        },

        /**
         * Contains methods related to the #alarm page
         * @namespace
         */
        alarm: {
            init: function UI_alarm_init() {
                $('#customDuration').val('').data('placeholder', '0');
            },

            setValue: function UI_alaram_setValue(duration) {
                var checkedRadios = $('#new_alarm input[type=radio]:checked'),
                    radios = $('#new_alarm input[type=radio]'),
                    radiosLen = radios.length,
                    radio = null,
                    i = 0;

                if (duration === undefined) {
                    duration = 0;
                }

                app.setAlarm(duration);

                if (checkedRadios.length > 0 &&
                    checkedRadios[0].id === 'yes_1') {
                    $('#yes_1').attr('checked', true).checkboxradio('refresh');
                    $('#customDuration').val(duration);
                    $('#customDuration')
                        .removeClass('ui-disabled').prop('disabled', false)
                        .trigger('focus');
                } else {
                    $('#customDuration')
                        .val('')
                        .addClass('ui-disabled')
                        .data('placeholder', '0');
                    for (i = 0; i < radiosLen; i += 1) {
                        radio = radios[i];
                        if (Number(radio.value) === duration) {
                            radio.checked = true;
                        } else {
                            radio.checked = false;
                        }
                        tau.widget.Checkboxradio(radio).refresh();
                    }
                }
                return duration;
            },

            getValue: function UI_alarm_getValue() {
                var value = parseInt($('#new_alarm input[type=radio]:checked')
                    .val(), 10);
                if (isNaN(value)) {
                    value = Math.abs(parseInt($('#customDuration').val(), 10));
                    if (isNaN(value)) {
                        value = 0;
                    }
                } else {
                    this.disableCustomDuration();
                }
                return value;
            },

            disableCustomDuration:
                function UI_alarm_disableCustomDuration() {
                    $('#customDuration').val('').addClass('ui-disabled')
                        .data('placeholder', '0');
                },

            getStoredValue: function UI_alarm_getStoredValue() {
                return parseInt($('#alarm').attr('data-minutes'), 10);
            },

            /**
             * Reads and sets alarm duration label.
             */
            updateDurationLabel: function UI_alarm_updateDurationLabel(value) {
                var unit, label;
                app.ui.alarm.setValue(value === 0 ? 0 : (value || -1));
                $('#alarm').attr('data-minutes', value);
                if (value === -1) {
                    label = 'Off';
                } else if (value === 0) {
                    label = 'On time';
                } else {
                    unit = 'minute';
                    if (value % 10080 === 0) {
                        value /= 10080;
                        unit = 'week';
                    } else if (value % 1440 === 0) {
                        value /= 1440;
                        unit = 'day';
                    } else if (value % 60 === 0) {
                        value /= 60;
                        unit = 'hour';
                    }
                    label = value + ' ' + unit + (value > 1 ? 's' : '') +
                        ' before';
                }
                $('#alarm').text(label);
            }
        },

        /**
         * Contains methods related to the new event page
         * @namespace
         */
        new_event: {
            /**
             * Indicates if new_event page has been initialized.
             * @type {bool}
             */
            initialized: false,

            /**
             * Animate status.
             * @type {bool}
             */
            animateStatus: false,

            /**
             * All-day element.
             * @type {jQuery}
             */
            allday: null,

            /**
             * Start element.
             * @type {jQuery}
             */
            start: null,

            /**
             * End element.
             * @type {jQuery}
             */
            end: null,

            /**
             * Popup element.
             * @type {jQuery}
             */
            popup: null,

            /**
             * Initializes the new_event page.
             */
            init: function Ui_newEvent_init() {
                this.assignFields();
                this.updateDateFormat();
                this.setStartDate();
                this.setEndDate();
                this.addEvents();
                this.initialized = true;
            },

            /**
             * Selects fields for future use.
             */
            assignFields: function Ui_newEvent_assignFields() {
                this.allday = $('#allDay');
                this.start = $('#demo-date-1');
                this.end = $('#demo-date-2');
                this.popup = $('#popup');
            },

            /**
             * Bind events.
             */
            addEvents: function Ui_newEvent_addEvents() {
                var self = this;

                this.allday.change(this.updateDateFormat.bind(this));
                this.start.change(this.validStart.bind(this));
                this.end.change(this.validEnd.bind(this));

                $('#new_event').on('pageshow', function onNewEventShow() {
                    self.animateStatusChange.bind(self)();
                });

                /* old events */
                $('#add-event-btn').on('click', this.addEvent.bind(this));
                $('#add-event-cancel-btn').on('click', this.cancel.bind(this));
                // alarm selection confirm
                $('#add-alarm').on('click', app.switchAlarm.bind(app));
                // go to alarm selection
                $('#set-alarm').on('click', function onSetAlarmClick() {
                    app.ui.alarm.setValue(app.ui.alarm.getStoredValue());
                    $.mobile.changePage('#new_alarm');
                });

                $(window).resize(function onResize() {
                    if ($.mobile.activePage.attr('id') === 'new_alarm') {
                        setTimeout(function onNewAlarmResize() {
                            var content = $('#new_alarm [data-role=content]'),
                                customDuration = $('#customDuration'),
                                size = customDuration.position().top +
                                    customDuration.outerHeight();
                            content.scrollview('scrollTo', 0, -1 * size, 0);
                        }, 260);
                    }
                });
            },
            /**
             * Update animate status.
             */
            animateStatusChange:
                function Ui_newEvent_animateStatusChange() {
                    var self = this;
                    this.end.next().find('span').not('.ui-datefield-seperator')
                        .off('click')
                        .on('click', function updateAnimateStatus(e) {
                            self.animateStatus =
                                $(e.target).hasClass('ui-btn-picker');
                        });
                },

            /**
             * Handles start date update.
             *
             * If start date is later than end date, then the end date is set
             * to the same value.
             */
            validStart: function Ui_newEvent_validStart() {
                this.setStartDate();
                if (this.startDate > this.endDate) {
                    this.end.datetimepicker('value', this.startDate);
                    this.setEndDate();
                }
            },

            /**
             * Handles end date update and prevents from setting the end date
             * which is earlier than the start date.
             */
            validEnd: function Ui_newEvent_validEnd() {
                if (this.startDate > this.getDateFromPicker(this.end)) {
                    this.setDateValue(this.end, this.endDate);
                    this.showWarning(
                        'End date cannot be earlier than initial date.',
                        function datetimepicker() {
                            $('#demo-date-2').datetimepicker();
                        }
                    );
                } else {
                    this.setEndDate();
                }
            },

            /**
             * Verifies start and end dates.
             */
            validAll: function Ui_newEvent_validAll() {
                this.validStart();
                this.validEnd();
            },

            /**
             * Shows a popup with warning.
             * @param {string} text
             * @param {function} successCallback
             */
            showWarning:
                function Ui_newEvent_showWarning(text, successCallback) {
                    if (successCallback instanceof Function) {
                        $('#popup').one('popupafterclose', successCallback);
                    }
                    this.lockTabKey();
                    app.ui.popup(text, {
                        'OK': function OK() {
                            $('#popup').popup('close');
                        }
                    });
                    this.popup.one('popupafterclose', function afterClose() {
                        $(document).off('keydown');
                    });
                },

            /**
             * Locks the tab key.
             */
            lockTabKey: function Ui_newEvent_lockTabKey() {
                $(document).on('keydown', function onKeyDown(event) {
                    if (event.keyCode === 9) {
                        event.preventDefault();
                    }
                });
            },

            /**
             * Updates date format.
             * @param {bool} fast If fast then some actions are omitted.
             */
            updateDateFormat: function Ui_newEvent_updateDateFormat(fast) {
                var time = tizen.time.getTimeFormat();
                if (this.allday.val() === '1') {
                    this.format = 'MMM dd yyyy';
                } else {
                    if (time === 'h:m:s') {
                        this.format = 'MMM dd yyyy HH:mm';
                    } else {
                        this.format = 'MMM dd yyyy hh:mm tt';
                    }
                }
                if (fast) {
                    this.start.datetimepicker('option', 'format', this.format);
                    this.end.datetimepicker('option', 'format', this.format);
                } else {
                    $('#title').blur();
                    this.start.datetimepicker('option', 'format', this.format)
                        .datetimepicker();
                    this.end.datetimepicker('option', 'format', this.format)
                        .datetimepicker();
                }
                $('#demo-date-1').datetimepicker();
                $('#demo-date-2').datetimepicker();
            },

            /**
             * Sets date value.
             * @param {jQuery} field
             * @param {Date} date
             */
            setDateValue: function Ui_newEvent_setDateValue(field, date) {
                var sp = field.next().find('span')
                    .not('.ui-datefield-seperator');
                field.datetimepicker('value', date);
                if (this.animateStatus) {
                    if (sp.animationComplete) {
                        sp.animationComplete(function onAnimationComplete() {
                            setTimeout(function setDate() {
                                field.datetimepicker('value', date);
                            }, 700);
                            sp.off('webkitAnimationEnd');
                            sp.off('animationend');
                        });
                    }
                }
            },

            /**
             * Reads and sets start date.
             */
            setStartDate: function Ui_newEvent_setStartDate() {
                this.startDate = this.getDateFromPicker(this.start);
            },

            /**
             * Reads and sets end date.
             */
            setEndDate: function Ui_newEvent_setEndDate() {
                this.endDate = this.getDateFromPicker(this.end);
            },

            /**
             * Returns date read from the datetimepicker field.
             * @param {jQuery} field
             * @return {Date}
             */
            getDateFromPicker:
                function Ui_newEvent_getDateFromPicker(field) {
                    return new Date(field.data('datetimepicker').options.date);
                },

            /**
             * Select allday select option.
             * @param {string} value
             */
            setSelectAllDay: function Ui_newEvent_setSelectAllDay(value) {
                var self = app.ui.new_event,
                    allDay = document.getElementById('allDay');
                value = value ? 1 : 0;
                self.allday.find('option').attr('selected', false);
                self.allday.find('option[value="' + value + '"]')[0]
                    .selected = true;
                tau.widget.ToggleSwitch(allDay).refresh();
            },

            /**
             * Adds event.
             * @param {type} e
             */
            addEvent: function Ui_newEvent_addEvent(e) {
                function goHome() {
                    window.setTimeout(function goHomeDefer() {
                        $.mobile.changePage('#home');
                    }, 0);

                }

                e.preventDefault();
                e.stopPropagation();
                var button = $('#add-event-btn');
                if (!button.hasClass('disabled')) {
                    button.addClass('disabled');
                    if (app.eventId === 0) {
                        this.context.app.addEvent(goHome);
                    } else {
                        this.context.app.updateEvent(goHome);
                    }
                }
            },

            /**
             * Cancels adding new event.
             * @param {type} e
             */
            cancel: function Ui_newEvent_cancel(e) {
                e.preventDefault();
                e.stopPropagation();
                $.mobile.changePage('#home');
            }
        },

        /**
         * Fixes content height.
         */
        fixContentHeight: function Ui_fixContentHeight() {
            var contentHeight = screen.availHeight -
                $('div[data-role="header"]').outerHeight() -
                $('div[data-role="footer"]').outerHeight();
            $('div[data-role="content"]').css('height', contentHeight);
        },

        /**
         * Creates and opens a popup.
         * @param {string} text
         * @param {object} buttons
         */
        popup: function UI_popup(text, buttons) {
            var i, popup = $('#popup');

            if (!popup.hasClass('ui-popup')) {
                popup.popup();
            }

            if (!buttons) {
                buttons =  {
                    'OK': function OK() {
                        $('#popup').popup('close');
                    }
                };
            }

            $('.ui-popup-button-bg', popup).empty();

            popup[0].className = popup[0].className
                .replace(/\bcenter_basic.*?\b/g, '');
            popup.addClass('center_basic_' +
                           Object.keys(buttons).length + 'btn');

            for (i in buttons) {
                if (buttons.hasOwnProperty(i)) {
                    if (buttons[i]) {
                        $('<a/>')
                            .text(i)
                            .attr({'data-role': 'button',
                                   'data-inline': 'true'})
                            .bind('click', buttons[i])
                            .appendTo($('.ui-popup-button-bg', popup));
                    }
                }
            }
            $('.ui-popup-text p', popup).text(text);

            popup.trigger('create');
            popup.popup('open', {positionTo: 'window'});
        }
    };
}());
